﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace PshaisIndicators
{
    public class MACDv2 : IIndicatorScript
    {
        int longLength = 26;
        int shortLength = 10;
        int signalLength = 9;
        
        Core.MAType longMaType = Core.MAType.Sma,
        shortMaType = Core.MAType.Sma,
        signalMaType = Core.MAType.Ema;
        
        const string ZEROCROSS = "zero-cross";
        const string MACDCROSS = "macd-cross";
        const string MACDDIR = "macd-dir";
        const string HISTDIR = "hist-dir";
        
        string buyTrigger = ZEROCROSS;
        string sellTrigger = MACDCROSS;

        double macd = 0;
        double macdH = 0;
        double macdS = 0;
        double lastMacdH = 0;
        double lastMacd = 0;

        // Properties
        public string Name
        {
            get { return "pshai's MACD V2"; }
        }
        public int TicksBack
        {
        	get { return Core.MacdExtLookback(shortLength, shortMaType, longLength, longMaType, signalLength, signalMaType); }
        }

        // Setting fields
        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> inputField = new List<ScriptParameter>
            {
                new ScriptParameter("Short Period", ScriptParameterType.Integer, shortLength.ToString()),
                new ScriptParameter("Short MA Type", ScriptParameterType.String, shortMaType.ToString()),
                new ScriptParameter("Long Period", ScriptParameterType.Integer, longLength.ToString()),
                new ScriptParameter("Long MA Type", ScriptParameterType.String, longMaType.ToString()),
                new ScriptParameter("Signal Period", ScriptParameterType.Integer, signalLength.ToString()),
                new ScriptParameter("Signal MA Type", ScriptParameterType.String, signalMaType.ToString()),
                new ScriptParameter("Buy Trigger", ScriptParameterType.String, buyTrigger),
                new ScriptParameter("Sell Trigger", ScriptParameterType.String, sellTrigger)
            };

            return inputField;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
        	shortLength = ParseInt(parameters["Short Period"], shortLength);
        	longLength = ParseInt(parameters["Long Period"], longLength);
        	signalLength = ParseInt(parameters["Signal Period"], signalLength);
        	
        	shortMaType = ParseMAType(parameters["Short MA Type"]);
        	longMaType = ParseMAType(parameters["Long MA Type"]);
        	signalMaType = ParseMAType(parameters["Signal MA Type"]);
        	
        	buyTrigger = ParseString(parameters["Buy Trigger"], buyTrigger);
        	sellTrigger = ParseString(parameters["Sell Trigger"], sellTrigger);
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 1, Name = "Zero-line", HexLineColor = "#aaaaaa" },
                new ChartRecord() { ChartIndex = 1, Name = "MACD", HexLineColor = "#FF0000" },
                new ChartRecord() { ChartIndex = 1, Name = "Signal", HexLineColor = "#FFFF00" },
                new ChartRecord() { ChartIndex = 1, Name = "Historic", HexLineColor = "#FF00FF" }
            };

            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>
            {
                0m,
                Convert.ToDecimal(macd),
                Convert.ToDecimal(macdS),
                Convert.ToDecimal(macdH)
            };

            // context.Logger.LogToFile(string.Format("Called short {0} and long {1}", ma_short, ma_long));

            return chartData;
        }

        // This is called as the script is loaded the first time, aka the init
        public void Init()
        {
            // In this code example we do not have to do anything
        }

        // This is the brain so the say, this is where the calculation is done and an output must be produced.
        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument; // Its pre-created for speed based on the "TicksBack" property field
                int position = instrument.Close.Count() - (1 + context.Offset); // minus 1 because of counting from 0 + account for the backtest

                // Lets do the calculation
                int beginIndex, outNBElements;
                double[] macdValues = new double[position];
                double[] macdSValues = new double[position];
                double[] macdHValues = new double[position];


                var macdReturnCode = Core.MacdExt(0,
		                                       position,
		                                       instrument.Close,
		                                       shortLength,
		                                       shortMaType,
		                                       longLength,
		                                       longMaType,
		                                       signalLength,
		                                       signalMaType,
		                                       out beginIndex,
		                                       out outNBElements,
		                                       macdValues,
		                                       macdSValues,
		                                       macdHValues
		                                      );
                //var EMAShortReturnCode = TicTacTec.TA.Library.Core.Sma(position, position, instrument.Close, shortLength, out beginIndex, out outNBElements, EMA_Short);
                //var EMALongReturnCode = TicTacTec.TA.Library.Core.Sma(position, position, instrument.Close, longLength, out beginIndex, out outNBElements, EMA_Long);

                // Verification
                if (macdReturnCode == TicTacTec.TA.Library.Core.RetCode.Success)
                {
                    if (outNBElements > 0)
                    {
                    	lastMacd = macd;
                		lastMacdH = macdH;
                		
                        macd = macdValues[outNBElements - 1];
                        macdS = macdSValues[outNBElements - 1];
                        macdH = macdHValues[outNBElements - 1];
                    }
                }

                
                if( sellTrigger.Contains(ZEROCROSS)
                   && macd < 0)
                	return IndicatorResult.SellShort;
                
                if( sellTrigger.Contains(MACDCROSS)
                   && macd < macdS)
                	return IndicatorResult.SellShort;
                
                if( sellTrigger.Contains(MACDDIR)
                   && macd < lastMacd)
                	return IndicatorResult.SellShort;
                
                if( sellTrigger.Contains(HISTDIR)
                   && macdH < lastMacdH)
                	return IndicatorResult.SellShort;
                
                
                
                if( buyTrigger.Contains(ZEROCROSS)
                   && macd > 0)
                	return IndicatorResult.BuyLong;
                
                if( buyTrigger.Contains(MACDCROSS)
                   && macd > macdS)
                	return IndicatorResult.BuyLong;
                
                if( buyTrigger.Contains(MACDDIR)
                   && macd > lastMacd)
                	return IndicatorResult.BuyLong;
                
                if( buyTrigger.Contains(HISTDIR)
                   && macdH > lastMacdH)
                	return IndicatorResult.BuyLong;
                
                
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex.Message));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }

		public static Core.MAType ParseMAType(object value)
		{
			Core.MAType val;
			switch( value.ToString().ToLower() )
			{
				case "sma":
					val = Core.MAType.Sma;
					break;
               	case "ema":
					val = Core.MAType.Ema;
					break;
               	case "dema":
					val = Core.MAType.Dema;
					break;
               	case "tema":
					val = Core.MAType.Tema;
					break;
               	case "trima":
					val = Core.MAType.Trima;
					break;
               	case "t3":
					val = Core.MAType.T3;
					break;
               	case "mama":
					val = Core.MAType.Mama;
					break;
               	case "kama":
					val = Core.MAType.Kama;
					break;
				case "wma":
					val = Core.MAType.Wma;
					break;
				default:
					val = Core.MAType.Sma;
					break;
			}
			
			return val;
		}
        
        int ParseInt(object value, int lastValue)
	    {
	    	try {
	    		return Convert.ToInt32(value.ToString());
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    double ParseDouble(object value, double lastValue)
	    {
	    	var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			try {
				double output;
				double.TryParse(
					value.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
				return output;
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    string ParseString(object value, string lastValue)
	    {
	    	try {
	    		return Convert.ToString(value);
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
    }
}
